/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * main.c
 * Copyright (C) Andrea Del Signore 2006 <sejerpz@tin.it>
 * 
 * main.c is free software.
 * 
 * You may redistribute it and/or modify it under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * main.c is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with main.c.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include <string.h>

#include <panel-applet.h>
#include "ovc-applet.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

OvcApplet *ovc_applet = NULL;

static void ovc_applet_destroy()
{
	g_object_unref(ovc_applet);
	ovc_applet = NULL;
}

static void
ovc_applet_delete(GtkWidget *applet, gpointer data)
{
	ovc_applet_destroy();
}

static gboolean
ovc_applet_fill (PanelApplet *applet,
				   const gchar *iid,
				   gpointer data)
{
	if (strcmp (iid, "OAFIID:GNOME_ovpnClient") != 0)
		return FALSE;

	if (ovc_applet != NULL)
	{
		/* Application already initialized!
		 * A destroy event? No sign in the docs!
		 * Free memory and widgets.
		 * TODO: need to investigate
		 */
		ovc_applet_destroy();
		return FALSE;
	}
	
	ovc_applet = ovc_applet_new(applet);
	g_signal_connect (G_OBJECT(ovc_applet), "delete_event", 
					  G_CALLBACK (ovc_applet_delete), NULL);	
	return TRUE;
}

PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_ovpnClient_Factory",
                             PANEL_TYPE_APPLET,
                             _("OpenVpn connection manager"),
                             "0",
                             ovc_applet_fill,
                             NULL);
