/***************************************************************************
 *            ovc-applet-log-window.c
 *
 *  Sat Jul  1 09:51:32 2006
 *  Copyright  2006  Andrea Del Signore
 *  sejerpz@tin.it
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glade/glade.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "ovc-applet-log-window.h"
#include "ovc-applet.h"
#include "ovc-connection.h"

static GladeXML *gxml;

GtkWidget *combo_widget;
GtkTextTagTable *tag_table;

static void ovc_send_command();

GtkWidget*
ovc_create_log_window (const gchar *glage_file)
{
	GtkWidget *window;
	GtkTextTag *tag;
	
	gxml = glade_xml_new (glage_file, "windowLog", NULL);
	
	/* This is important */
	glade_xml_signal_autoconnect (gxml);
	window = glade_xml_get_widget (gxml, "windowLog");
	
	tag_table = gtk_text_tag_table_new();
	tag = gtk_text_tag_new("information");

	g_object_set(GTK_TEXT_TAG(tag),
				 "foreground", "DimGray", 
				 "style", PANGO_STYLE_ITALIC, 
				 "weight", PANGO_WEIGHT_BOLD,
				 NULL);
	
	gtk_text_tag_table_add(tag_table, tag);
	
	tag = gtk_text_tag_new("message");
	g_object_set(GTK_TEXT_TAG(tag),"style", PANGO_STYLE_NORMAL,NULL);
	gtk_text_tag_table_add(tag_table, tag);
	
	tag = gtk_text_tag_new("connection-name");
	g_object_set(GTK_TEXT_TAG(tag),"weight", PANGO_WEIGHT_BOLD,NULL);
	gtk_text_tag_table_add(tag_table, tag);
	
	GtkTextBuffer *textBuffer =  gtk_text_buffer_new(tag_table);
	GtkTextView *textWidget = (GtkTextView *)glade_xml_get_widget(gxml,"textviewBuffer");
	gtk_text_view_set_buffer(textWidget, textBuffer);
	
	combo_widget = glade_xml_get_widget(gxml, "comboboxConnection");
	GtkListStore *store;
	store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
    GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo_widget), renderer, TRUE);
    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo_widget), renderer,
                                   "text", 0, NULL);
	gtk_combo_box_set_model(GTK_COMBO_BOX(combo_widget), (GtkTreeModel *)store);

	GtkWidget *button = glade_xml_get_widget(gxml, "buttonSend");
	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);

	g_signal_connect (G_OBJECT (window), "delete-event",
			  G_CALLBACK (gtk_widget_hide_on_delete), NULL);

	return window;
}


gboolean
on_entrySend_key_press_event(GtkWidget *widget,GdkEventKey *event, gpointer data)
{
	if (event->keyval == GDK_Return)
	{
		ovc_send_command();
		return TRUE;
	}
	return FALSE;
}

static void
ovc_send_command()
{
	GtkEntry *entry=(GtkEntry *) glade_xml_get_widget(gxml, "entrySend");
	gchar *message = g_strdup_printf("%s\n", gtk_entry_get_text(entry));
	GtkTreeIter iter;
	
	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo_widget), &iter))
	{
		GtkTreeModel *store = GTK_TREE_MODEL(gtk_combo_box_get_model(GTK_COMBO_BOX(combo_widget)));
		OvcConnection *conn = NULL;
		
		gtk_tree_model_get(store, &iter, 1, &conn, -1);
		ovc_connection_write(conn, message);
	}
	g_free(message);
}


void
on_comboboxConnection_changed(GtkWidget *widget, gpointer data)
{
	GtkComboBox *combo = GTK_COMBO_BOX(widget);
	GtkWidget *send = glade_xml_get_widget(gxml,"buttonSend");
	
	if (gtk_combo_box_get_active(combo) == -1)
		gtk_widget_set_sensitive(send, FALSE);
	else
		gtk_widget_set_sensitive(send, TRUE);
}

void
on_buttonSend_clicked(GtkWidget *widget, gpointer data)
{
	ovc_send_command();
}


void
ovc_combo_box_remove_connection(OvcConnection *connection)
{
	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(combo_widget)));
	GtkTreeIter iter;
	gboolean valid;

	/* Get the first iter in the list */
	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);

	while (valid)
    {
		OvcConnection *conn;
		
		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 1, &conn,-1);
		if (conn == connection)
		{
			gtk_list_store_remove(store, &iter);
			if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
				gtk_combo_box_set_active_iter(GTK_COMBO_BOX(combo_widget), &iter);
			else
				gtk_combo_box_set_active(GTK_COMBO_BOX(combo_widget), -1);
			
			break;
		}
		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL(store), &iter);
    }	
}
	
void 
ovc_log_window_append_message(LOG_MESSAGE_TYPES message_type, gchar *connection, gchar *message)
{
	GtkTextIter iter, end_iter;
	GtkTextView *textWidget = (GtkTextView *)glade_xml_get_widget(gxml,"textviewBuffer");
	if (textWidget == NULL)
		return;
	
	GtkTextBuffer *textBuffer = gtk_text_view_get_buffer(textWidget);
	GtkTextTag *tag = NULL;
	GtkTextTag *tag_connection = NULL;
	
	tag_connection = gtk_text_tag_table_lookup(tag_table, "connection-name");
	
	gtk_text_buffer_get_end_iter (textBuffer, &iter);
	switch(message_type)
	{
	case LOG_MESSAGE_NORMAL:
		tag = gtk_text_tag_table_lookup(tag_table, "message");
	break;
	case LOG_MESSAGE_INFO:
		tag = gtk_text_tag_table_lookup(tag_table, "information");
	break;
	default:
		tag = gtk_text_tag_table_lookup(tag_table, "message");
		break;
	}
	
	gchar **lines = g_strsplit(message, "\n",2);
	gchar *line;
	
	gchar *tmp;
	gint line_count = gtk_text_buffer_get_line_count(textBuffer);
	
	while ( (line = lines[0]) != NULL)
	{
		if (line_count > 500)
		{
			gtk_text_buffer_get_start_iter(textBuffer, &iter);
			gtk_text_buffer_get_iter_at_line(textBuffer, &end_iter, 1);
			gtk_text_buffer_delete(textBuffer, &iter, &end_iter);
			gtk_text_buffer_get_end_iter (textBuffer, &iter);
			line_count--;
		}
		
		if (connection != NULL)
		{
			tmp = g_strdup_printf("%s: ", connection);
			gtk_text_buffer_insert_with_tags(textBuffer, &iter, tmp, -1, tag_connection,NULL);
			g_free(tmp);
		}
		tmp = g_strdup_printf("%s\n", line);
		gtk_text_buffer_insert_with_tags(textBuffer, &iter, tmp, -1, tag,NULL);	
		g_free(tmp);
		line_count++;
		
		if (lines[1] != NULL)
		{
			tmp = g_strdup(lines[1]);
			g_strfreev(lines);
			lines = g_strsplit(tmp, "\n",2);
			g_free(tmp);
		}
		else
			break;
	} 


	GtkTextMark *mark = gtk_text_buffer_create_mark(textBuffer, NULL, &iter, FALSE);
	gtk_text_view_scroll_to_iter(textWidget, &iter, 0, FALSE, 0,0);
	gtk_text_view_scroll_to_mark (textWidget, mark, 0, FALSE, 0,0);
	gtk_text_buffer_delete_mark(textBuffer, mark);
}

void 
ovc_combo_box_connection_status_changing(OvcConnection *connection, OVC_CONNECTION_STATES status)
{
	gchar *name = NULL;
	//OVC_CONNECTION_STATES prevstatus = OVC_CONNECTION_STATUS_UNKNOWN;
	
	
	g_object_get(G_OBJECT(connection), "name", &name, NULL);
	
	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(combo_widget)));
	GtkTreeIter iter;
	
	if (status != OVC_CONNECTION_STATUS_UNKNOWN &&
		status != OVC_CONNECTION_STATUS_INITIALIZING &&
		status != OVC_CONNECTION_STATUS_QUERYING)
	{
		gtk_list_store_append(store, &iter);
	    gtk_list_store_set(store, &iter,
                               0, name,
						       1, connection, -1);		
		if (gtk_combo_box_get_active (GTK_COMBO_BOX(combo_widget)) == -1)
			gtk_combo_box_set_active_iter(GTK_COMBO_BOX(combo_widget), &iter);
	}
	else
	{
		ovc_combo_box_remove_connection(connection);
	}
	
	g_free(name);
	
}
