/***************************************************************************
 *            ovc-applet.c
 *
 *  Sat Jul  1 09:01:54 2006
 *  Copyright  2006  Andrea Del Signore
 *  sejerpz@tin.it
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <panel-applet.h>
#include <libgnomevfs/gnome-vfs.h>
#include <libgnome/libgnome.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <libnotify/notification.h>
#include <libnotify/notify.h>


#include "ovc-connection.h"
#include "ovc-connections.h"
#include "ovc-ui-connections.h"
#include "ovc-about.h"
#include "ovc-applet.h"
#include "ovc-applet-log-window.h"

#define GLADE_FILE PACKAGE_DATA_DIR"/ui/ovpnclient.glade"
#define ICON_APPLICATION PACKAGE_DATA_DIR"/pixmaps/ovpnclient.png"
#define ICON_APPLICATION_PANEL PACKAGE_DATA_DIR"/pixmaps/ovpnclient-panel.png"
#define ICON_STATUS_UNKNOWN PACKAGE_DATA_DIR"/pixmaps/unknown.png"
#define ICON_STATUS_CONNECTED PACKAGE_DATA_DIR"/pixmaps/connected.png"
#define ICON_STATUS_NOT_CONNECTED PACKAGE_DATA_DIR"/pixmaps/notconnected.png"

#define ICON_NOTIFY_STATUS_CONNECTED PACKAGE_DATA_DIR"/pixmaps/notify-connected.png"
#define ICON_NOTIFY_STATUS_NOT_CONNECTED PACKAGE_DATA_DIR"/pixmaps/notify-notconnected.png"



static void ovc_applet_class_init(OvcAppletClass *klass);
static void ovc_applet_init(OvcApplet *sp);
static void ovc_applet_dispose(GObject *object);
static void ovc_applet_finalize(GObject *object);
static void ovc_connections_get_property (GObject * object,
										  guint property_id,
										  GValue * value, GParamSpec * pspec);

static void ovc_connections_set_property (GObject * object,
										  guint property_id,
										  const GValue * value, GParamSpec * pspec);

static void ovc_intialize_applet_ui(OvcApplet *applet);

static gboolean ovc_select_connection(GtkWidget *widget, GdkEventButton *event, gpointer data);

static void ovc_properties_dialog_show (BonoboUIComponent *uic,gpointer data, const gchar *verbname);
static void ovc_log_dialog_show (BonoboUIComponent *uic, gpointer data, const gchar *verbname);
static void ovc_about_dialog_show (BonoboUIComponent *uic, gpointer data, const gchar *verbname);

static void
ovc_connections_list_clear(OvcConnections *sender, gpointer data);
static void
ovc_connections_item_removed(OvcConnections *sender, OvcConnection *connection, gpointer data);
static void
ovc_connections_item_added(OvcConnections *sender, OvcConnection *connection, gpointer data);

static void ovc_menu_connections_build(OvcApplet *applet);

static void ovc_connection_status_changing(OvcConnection *connection, 
										   OVC_CONNECTION_STATES status, 
										   gpointer   data);

static const BonoboUIVerb right_click_menu_verbs [] = {
	BONOBO_UI_VERB (_("Properties"), (BonoboUIVerbFn) ovc_properties_dialog_show),
	BONOBO_UI_VERB (_("Log"), (BonoboUIVerbFn) ovc_log_dialog_show),
	BONOBO_UI_VERB (_("About"), (BonoboUIVerbFn) ovc_about_dialog_show),
	BONOBO_UI_VERB_END
};

struct _OvcAppletPrivate {
	/* Place Private Members Here */
	PanelApplet *applet;
	GtkWidget *event_box;
	GtkWidget *log_window;
	GtkWidget *combo_widget;
	
	GtkWidget *menu_connections;
	OvcConnections *connections;
	gboolean disposed;
};

typedef struct _OvcAppletSignal OvcAppletSignal;
typedef enum _OvcAppletSignalType OvcAppletSignalType;

enum _OvcAppletSignalType {
	/* Place Signal Types Here */
	SIGNAL_TYPE_DELETE_EVENT,
	LAST_SIGNAL
};

struct _OvcAppletSignal {
	OvcApplet *object;
};

static guint ovc_applet_signals[LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

static const char right_click_menu_xml [] =
   "<popup name=\"button3\">\n"
   "   <menuitem name=\"Properties Item\" "
   "             verb=\"Properties\" "
   "           _label=\"_Preferences...\"\n"
   "          pixtype=\"stock\" "
   "          pixname=\"gtk-properties\"/>\n"
   "   <menuitem name=\"Log Window\" "
   "             verb=\"Log\" "
   "           _label=\"_Show log...\"\n"
   "          pixtype=\"stock\" "
   "          pixname=\"gtk-info\"/>\n"
   "   <menuitem name=\"About Item\" "
   "             verb=\"About\" "
   "           _label=\"_About...\"\n"
   "          pixtype=\"stock\" "
   "          pixname=\"gnome-stock-about\"/>\n"
   "</popup>\n";

GType
ovc_applet_get_type()
{
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo our_info = {
			sizeof (OvcAppletClass),
			NULL,
			NULL,
			(GClassInitFunc)ovc_applet_class_init,
			NULL,
			NULL,
			sizeof (OvcApplet),
			0,
			(GInstanceInitFunc)ovc_applet_init,
		};

		type = g_type_register_static(G_TYPE_OBJECT, 
			"OvcApplet", &our_info, 0);
	}

	return type;
}

static void
ovc_connections_set_property (GObject * object,
							 guint property_id,
							 const GValue * value, GParamSpec * pspec)
{
	OvcApplet *self = OVC_APPLET(object);

	switch (property_id)
	{
		case OVC_APPLET_CONSTRUCT_APPLET_OBJECT:
			self->priv->applet = g_value_get_object(value);
		    g_object_ref(G_OBJECT(self->priv->applet));
		break;
		default:
			/* We don't have any other property... */
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
ovc_connections_get_property (GObject * object,
							 guint property_id,
							 GValue * value, GParamSpec * pspec)
{
	OvcApplet *self = (OvcApplet *) object;

	switch (property_id)
	{
		case OVC_APPLET_CONSTRUCT_APPLET_OBJECT:
			g_value_set_object (value, self->priv->applet);
		break;
		default:
			/* We don't have any other property... */
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
ovc_applet_class_init(OvcAppletClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS(klass);
	GParamSpec *param_spec;
	GType param_types[1];
	
	parent_class = g_type_class_peek_parent(klass);
	object_class->finalize = ovc_applet_finalize;
	object_class->dispose = ovc_applet_dispose;
	
	/* Create signals here */
	param_types[0] = G_TYPE_NONE;
	ovc_applet_signals[SIGNAL_TYPE_DELETE_EVENT] =   g_signal_newv("delete-event",
                 G_TYPE_FROM_CLASS (object_class),
                 G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
                 NULL /* class closure */,
                 NULL /* accumulator */,
                 NULL /* accu_data */,
                 g_cclosure_marshal_VOID__VOID,
                 G_TYPE_NONE /* return_type */,
                 0     /* n_params */,
	                   /* param_types */
                 param_types);
	
	/* Installing properties handlers */
	object_class->set_property = ovc_connections_set_property;
	object_class->get_property = ovc_connections_get_property;
	
	param_spec = g_param_spec_object ("applet",
									   "PanelApplet object",
									   "PanelApplet object",
									   G_TYPE_OBJECT,
									   G_PARAM_CONSTRUCT |
									   G_PARAM_READWRITE);
	
	g_object_class_install_property (object_class,
									 OVC_APPLET_CONSTRUCT_APPLET_OBJECT,
									 param_spec);

}

static void
ovc_applet_init(OvcApplet *obj)
{
	obj->priv = g_new0(OvcAppletPrivate, 1);
	/* Initialize private members, etc. */
	obj->priv->disposed = FALSE;
}

static void
ovc_applet_dispose(GObject *object)
{
	OvcApplet *cobj;
	cobj = OVC_APPLET(object);
	
	/* Free private members, etc. */
	if (cobj->priv->disposed)
		return;
	
	cobj->priv->disposed = TRUE;
	g_object_unref(cobj->priv->applet);
	g_object_unref(cobj->priv->connections);
	notify_uninit();
	gtk_widget_destroy(cobj->priv->event_box);

	G_OBJECT_CLASS(parent_class)->dispose(object);
}

static void
ovc_applet_finalize(GObject *object)
{
	OvcApplet *cobj;
	cobj = OVC_APPLET(object);
	
	/* Free private members, etc. */
		
	g_free(cobj->priv);
	G_OBJECT_CLASS(parent_class)->finalize(object);
}

OvcApplet *
ovc_applet_new(PanelApplet *applet)
{
	OvcApplet *obj;
	
	obj = OVC_APPLET(g_object_new(OVC_TYPE_APPLET, "applet", applet, NULL));
	
	ovc_intialize_applet_ui(obj);
	return obj;
}

static gboolean
ovc_applet_delete(GtkWidget *widget, gpointer data)
{
	OvcApplet *cobj = OVC_APPLET(data);
	g_signal_emit(OVC_APPLET(cobj), 
				  ovc_applet_signals[SIGNAL_TYPE_DELETE_EVENT],
				  0);
	return TRUE;
}

static void
ovc_intialize_applet_ui (OvcApplet *applet)
{
	
	notify_init("ovpnClient");

	/* Main Applet UI */
	GtkWidget *image = gtk_image_new_from_file (ICON_APPLICATION_PANEL);

	applet->priv->event_box = gtk_event_box_new ();
	gtk_container_add (GTK_CONTAINER (applet->priv->event_box), image);
	g_signal_connect(G_OBJECT(applet->priv->event_box), 
					 "delete-event", 
					 G_CALLBACK(ovc_applet_delete), 
					 (gpointer) applet);
	
	g_signal_connect(G_OBJECT (applet->priv->event_box), 
                  "button_press_event",
                  G_CALLBACK (ovc_select_connection),
				  (gpointer) applet);
					  
	gtk_container_add (GTK_CONTAINER (applet->priv->applet), applet->priv->event_box);
	
	/* Custom rigth click menu */
	panel_applet_setup_menu (PANEL_APPLET (applet->priv->applet),
                         right_click_menu_xml,
                         right_click_menu_verbs,
                         applet);
	
	/* Init config */
	if (gnome_vfs_initialized() != TRUE)
		gnome_vfs_init();
	
	/* Show interface */
	gtk_widget_show_all (GTK_WIDGET (applet->priv->applet));
	
	/* LOG WINDOW */
	applet->priv->log_window = ovc_create_log_window(GLADE_FILE);

	/* CONNECTIONS */
	gchar *configfile = gnome_util_home_file("ovpnclient.xml");
	
	applet->priv->connections = ovc_connections_new();
	g_signal_connect (G_OBJECT (applet->priv->connections), "list-clear", 
					  G_CALLBACK (ovc_connections_list_clear), (gpointer) applet);
	g_signal_connect (G_OBJECT (applet->priv->connections), "item-added", 
					  G_CALLBACK (ovc_connections_item_added), (gpointer) applet);
	g_signal_connect (G_OBJECT (applet->priv->connections), "item-removed", 
					  G_CALLBACK (ovc_connections_item_removed), (gpointer) applet);
	ovc_connections_load(applet->priv->connections, configfile);	
}

static gboolean
ovc_select_connection(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	OvcApplet *applet = OVC_APPLET(data);
	
	if (event->type == GDK_BUTTON_PRESS && event->button == 1)
	{
		applet->priv->menu_connections = gtk_menu_new ();
		ovc_menu_connections_build(applet);
		gtk_menu_popup (GTK_MENU (applet->priv->menu_connections), NULL, NULL, NULL, NULL,
							0, gtk_get_current_event_time());
		return TRUE;
	}
	return FALSE;
}



static void 
ovc_data_received(OvcConnection *connection,gchar *message, gpointer   data )
{
	gchar *name = NULL;
	g_object_get(connection, "name", &name, NULL);
	ovc_log_window_append_message(LOG_MESSAGE_NORMAL,name, message);
	g_free(name);
}

static void
ovc_menu_item_response(GtkWidget *menu, gpointer data)
{
	OvcConnection *connection = OVC_CONNECTION(data);
	
	gchar *name = NULL, *buff;
	OVC_CONNECTION_STATES status;
	
	g_object_get(connection,"name", &name, "status", &status, NULL);

	buff = g_strdup(_("open connection\n"));
	ovc_log_window_append_message(LOG_MESSAGE_INFO, name, buff);
	
	if (status == OVC_CONNECTION_STATUS_NOT_CONNECTED)
		ovc_connection_open(connection);
	else
		ovc_connection_close(connection);
	g_free(name);
	g_free(buff);
}

static void
ovc_menu_connections_build(OvcApplet *applet)
{
	GList *connection_list = NULL;
	GtkWidget *menu_image;
	GList *curr;
	
	g_object_get(G_OBJECT(applet->priv->connections), "items", &connection_list, NULL);
	for(curr=connection_list; curr != NULL; curr = curr->next)
	{
		OvcConnection *connection = (OvcConnection *) curr->data;
		OVC_CONNECTION_STATES status;
		gchar *name;
	
		g_object_get(G_OBJECT(connection), "status", &status, "name", &name, NULL);
		
		/* Create a new menu-item for the connection */
		GtkWidget *menu_item = gtk_image_menu_item_new_with_label (name);
		gtk_menu_shell_append (GTK_MENU_SHELL (applet->priv->menu_connections), menu_item);
		g_signal_connect (G_OBJECT (menu_item), "activate",
						  G_CALLBACK (ovc_menu_item_response), 
						  (gpointer) connection);
	
		gtk_widget_set_sensitive(GTK_WIDGET(menu_item), (status == OVC_CONNECTION_STATUS_CONNECTED ||
														 status == OVC_CONNECTION_STATUS_NOT_CONNECTED ? TRUE : FALSE));
		gtk_widget_show (menu_item);
		
		if (status == OVC_CONNECTION_STATUS_CONNECTED)
		{
			menu_image = gtk_image_new_from_file (ICON_STATUS_NOT_CONNECTED);		
		}
		else if (status == OVC_CONNECTION_STATUS_UNKNOWN)
		{
			menu_image = gtk_image_new_from_file (ICON_STATUS_UNKNOWN);
		}
		else
		{
			menu_image = gtk_image_new_from_file (ICON_STATUS_CONNECTED);
		}
		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), menu_image);
		g_free(name);
	}
}



static void
ovc_connections_list_clear(OvcConnections *sender, gpointer data)
{
	ovc_log_window_append_message(LOG_MESSAGE_INFO,NULL, "List clear\n");
}


static void
ovc_connections_item_removed(OvcConnections *sender, OvcConnection *connection, gpointer data)
{
	gchar *name=NULL, *message=NULL;
	g_object_get(connection,"name",&name,NULL);
	message = g_strdup(_("item removed\n"));
	ovc_log_window_append_message(LOG_MESSAGE_INFO, name, message);
	g_free(name);
	g_free(message);
	ovc_combo_box_remove_connection(connection);
	
}

static void
ovc_connections_item_added(OvcConnections *sender, OvcConnection *connection, gpointer data)
{
	gchar *name=NULL, *message=NULL;
	g_object_get(connection,"name",&name,NULL);
	message = g_strdup(_("item added\n"));

	
	/* Register connection signal callbacks */
	g_signal_connect (G_OBJECT (connection), "status-changing", G_CALLBACK (ovc_connection_status_changing), data);
	g_signal_connect (G_OBJECT (connection), "data-received", G_CALLBACK (ovc_data_received), data);

	OVC_CONNECTION_STATES status = OVC_CONNECTION_STATUS_UNKNOWN;
	
	g_object_get(G_OBJECT(connection),"status", &status, NULL);
	
	ovc_log_window_append_message(LOG_MESSAGE_INFO, name, message);
	g_free(name);
	g_free(message);
	ovc_connection_control_channel_open(connection);
	
}

static void 
ovc_connection_status_changing(OvcConnection *connection, OVC_CONNECTION_STATES status, gpointer   data)
{
	OvcApplet *applet = OVC_APPLET(data);
	gchar *name = NULL, *buff;
	OVC_CONNECTION_STATES prevstatus = OVC_CONNECTION_STATUS_UNKNOWN;
	NotifyNotification *n = NULL;
	
	g_object_get(G_OBJECT(connection), "name", &name, "status", &prevstatus, NULL);
	buff = g_strdup_printf(_("status changing from %s to %s\n"), gettext(status_descriptions[prevstatus]), gettext(status_descriptions[status]));
	ovc_log_window_append_message(LOG_MESSAGE_INFO, name, buff);
	g_free(buff);
	
	ovc_combo_box_connection_status_changing(connection, status);
	if (status == OVC_CONNECTION_STATUS_CONNECTED &&
		prevstatus != OVC_CONNECTION_STATUS_INITIALIZING)
	{
		n = notify_notification_new (name, 
									 _("connection established"),
									 ICON_NOTIFY_STATUS_CONNECTED, 
									 GTK_WIDGET(applet->priv->applet));

	}
	else if (status == OVC_CONNECTION_STATUS_NOT_CONNECTED && 
			 (prevstatus == OVC_CONNECTION_STATUS_CLOSING || 
			  prevstatus == OVC_CONNECTION_STATUS_CONNECTING) )
	{
		n = notify_notification_new (name, 
						 _("connection closed"),
						 ICON_NOTIFY_STATUS_NOT_CONNECTED, 
						GTK_WIDGET(applet->priv->applet));
	}
	if (n != NULL)
	{
		notify_notification_set_urgency(n, NOTIFY_URGENCY_NORMAL);
		notify_notification_set_timeout (n, NOTIFY_EXPIRES_DEFAULT); 

		if (!notify_notification_show (n, NULL)) 
			g_warning("failed to send notification\n");
		g_object_unref(G_OBJECT(n));
	}
	
	g_free(name);
}

static void
ovc_properties_dialog_show(BonoboUIComponent *uic,gpointer data, const gchar *verbname)
{
	OvcApplet *applet = OVC_APPLET(data);
	OvcUiConnections *dialog = ovc_ui_connections_new(GLADE_FILE,applet->priv->connections);
	
	ovc_ui_connections_show(dialog);
}

static void
ovc_log_dialog_show (BonoboUIComponent *uic, gpointer data, const gchar *verbname) 
{
	OvcApplet *self = OVC_APPLET(data);
	gtk_widget_show (self->priv->log_window);
}

static void
ovc_about_dialog_show (BonoboUIComponent *uic, gpointer data, const gchar *verbname) 
{
	ovc_about_show();
}
