/***************************************************************************
 *            ovc-ui-connection.c
 *
 *  Sat Jun 24 11:58:00 2006
 *  Copyright  2006  Andrea Del Signore
 *  sejerpz@tin.it
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glade/glade.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <errno.h>

#include "ovc-ui-connection.h"

static void ovc_ui_connection_class_init(OvcUiConnectionClass *klass);
static void ovc_ui_connection_init(OvcUiConnection *sp);
static void ovc_ui_connection_finalize(GObject *object);

struct _OvcUiConnectionPrivate {
	/* Place Private Members Here */
	gchar *ui;
	OvcConnection *connection;
	GtkWidget *widget;
	gint response;
	gboolean disposed;
	GladeXML *gxml;
};

typedef struct _OvcUiConnectionSignal OvcUiConnectionSignal;
typedef enum _OvcUiConnectionSignalType OvcUiConnectionSignalType;

enum _OvcUiConnectionSignalType {
	/* Place Signal Types Here */
	SIGNAL_TYPE_EXAMPLE,
	LAST_SIGNAL
};

struct _OvcUiConnectionSignal {
	OvcUiConnection *object;
};

//static guint ovc_ui_connection_signals[LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

GType
ovc_ui_connection_get_type()
{
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo our_info = {
			sizeof (OvcUiConnectionClass),
			NULL,
			NULL,
			(GClassInitFunc)ovc_ui_connection_class_init,
			NULL,
			NULL,
			sizeof (OvcUiConnection),
			0,
			(GInstanceInitFunc)ovc_ui_connection_init,
		};

		type = g_type_register_static(G_TYPE_OBJECT, 
			"OvcUiConnection", &our_info, 0);
	}

	return type;
}

static void
ovc_connection_set_property (GObject * object,
							 guint property_id,
							 const GValue * value, GParamSpec * pspec)
{
	OvcUiConnection *self = (OvcUiConnection *) object;

	switch (property_id)
	{
	case OVC_UICONNECTION_CONSTRUCT_CONNECTION:
		if (self->priv->connection != NULL)
			g_object_unref (OVC_CONNECTION(self->priv->connection));
		self->priv->connection = OVC_CONNECTION(g_value_dup_object(value));
	break;
	case OVC_UICONNECTION_CONSTRUCT_GLADE_UI:
		g_free(self->priv->ui);		
		self->priv->ui = g_value_dup_string(value);
	break;
	default:
		/* We don't have any other property... */
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
ovc_connection_get_property (GObject * object,
							 guint property_id,
							 GValue * value, GParamSpec * pspec)
{
	OvcUiConnection *self = (OvcUiConnection *) object;

	switch (property_id)
	{
	case OVC_UICONNECTION_CONSTRUCT_CONNECTION:
		g_value_set_object (value, self->priv->connection);
	break;
	case OVC_UICONNECTION_CONSTRUCT_GLADE_UI:
		g_value_set_string(value, self->priv->ui);
	break;
	default:
		/* We don't have any other property... */
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
	break;
	}
}

static void
ovc_ui_connection_dispose(GObject *object)
{
	OvcUiConnection *cobj;
	cobj = OVC_UICONNECTION(object);
	
	if (cobj->priv->disposed == TRUE)
		return;
	
	cobj->priv->disposed = TRUE;
	g_object_unref(cobj->priv->connection);
}

static void
ovc_ui_connection_class_init(OvcUiConnectionClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS(klass);
	GParamSpec *connection_param_spec;
	
	parent_class = g_type_class_peek_parent(klass);
	object_class->finalize = ovc_ui_connection_finalize;
	object_class->dispose = ovc_ui_connection_dispose;
	
	/* Create signals here:
	   ovc_ui_connection_signals[SIGNAL_TYPE_EXAMPLE] = g_signal_new(...)
 	*/
	
	object_class->set_property = ovc_connection_set_property;
	object_class->get_property = ovc_connection_get_property;

	connection_param_spec = g_param_spec_object("connection",
											 "Connections construct prop",
											 "Set Ovc connections object",
											 OVC_TYPE_CONNECTION ,
											 G_PARAM_CONSTRUCT_ONLY |
											 G_PARAM_READWRITE);

	g_object_class_install_property (object_class,
								 OVC_UICONNECTION_CONSTRUCT_CONNECTION,
								 connection_param_spec);
									 
	connection_param_spec = g_param_spec_string("glade-ui",
											 "Dialog widget construct prop",
											 "Set Ovc dialog widget object",
											"glade-ui-not-set",
											 G_PARAM_CONSTRUCT_ONLY |
											 G_PARAM_READWRITE);
											 
	g_object_class_install_property (object_class,
									 OVC_UICONNECTION_CONSTRUCT_GLADE_UI,
									 connection_param_spec);
}

static void
ovc_ui_connection_init(OvcUiConnection *obj)
{
	obj->priv = g_new0(OvcUiConnectionPrivate, 1);
	/* Initialize private members, etc. */
	obj->priv->disposed = FALSE;
}


static void
ovc_ui_connection_finalize(GObject *object)
{
	OvcUiConnection *cobj;
	cobj = OVC_UICONNECTION(object);
	
	/* Free private members, etc. */
	g_free(cobj->priv->ui);
	g_free(cobj->priv);
	G_OBJECT_CLASS(parent_class)->finalize(object);
}

static
void on_buttonConnectionOk_clicked(GtkWidget *widget, gpointer data)
{
	OvcUiConnection *self = OVC_UICONNECTION(data);
	
	/* Store connection configuration */
	OvcConnection *conn = self->priv->connection;
	GladeXML *gxml = self->priv->gxml;
	
	const gchar *name, *host, *port_text;
	GtkEntry *text = (GtkEntry *) glade_xml_get_widget(gxml,"entryConnectionName");
	name = gtk_entry_get_text(text);
	text = (GtkEntry *) glade_xml_get_widget(gxml,"entryConnectionHost");
	host = gtk_entry_get_text(text);
	text = (GtkEntry *) glade_xml_get_widget(gxml,"entryConnectionPort");
	port_text = gtk_entry_get_text(text);
	
	errno = 0;
	gint port = strtol(port_text, NULL, 0);
	if (errno)
		port = 0;
	
	g_object_set(conn, "name", name, "host", host, "port", port, NULL);
	

	gtk_dialog_response(GTK_DIALOG(self->priv->widget), GTK_RESPONSE_OK);
	gtk_widget_hide(GTK_WIDGET(self->priv->widget));
}

static
void on_buttonConnectionCancel_clicked(GtkWidget *widget, gpointer data)
{
	OvcUiConnection *self = OVC_UICONNECTION(data);
	
	gtk_dialog_response(GTK_DIALOG(self->priv->widget), GTK_RESPONSE_CANCEL);
}

static void
gtk_entry_set_text_if_null(GtkEntry *entry, const gchar *text)
{
	if (text != NULL)
		gtk_entry_set_text(entry, text);
	else
		gtk_entry_set_text(entry, "");
}

OvcUiConnection *
ovc_ui_connection_new(const gchar *glade_ui, OvcConnection *connection)
{
	OvcUiConnection *obj;
	
	obj = OVC_UICONNECTION(g_object_new(OVC_TYPE_UICONNECTION, "glade-ui", glade_ui, "connection", connection, NULL));

	/* UI */
	GladeXML *gxml = glade_xml_new(glade_ui, "dialogConnectionProperties", NULL);
	obj->priv->gxml = gxml;
	
	obj->priv->widget = glade_xml_get_widget(gxml, "dialogConnectionProperties");
	
	//g_signal_connect(G_OBJECT(dialog), "delete-event", G_CALLBACK(gtk_widget_hide_on_delete), (gpointer) obj);
	GtkButton *button = (GtkButton *)glade_xml_get_widget(gxml, "buttonConnectionOk");
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_buttonConnectionOk_clicked), (gpointer) obj);
	button = (GtkButton *)glade_xml_get_widget(gxml, "buttonConnectionCancel");
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(on_buttonConnectionCancel_clicked), (gpointer) obj);
	
	/* Populate UI with connections information */
	const gchar *name, *host, *port_text;
	gint port = 0;
	
	g_object_get(G_OBJECT(connection), "name", &name, "host", &host, "port", &port,NULL);
	port_text = g_strdup_printf("%d",port);
	
	GtkEntry *text = (GtkEntry *) glade_xml_get_widget(gxml,"entryConnectionName");
	gtk_entry_set_text_if_null(text, name);
	text = (GtkEntry *) glade_xml_get_widget(gxml,"entryConnectionHost");
	gtk_entry_set_text_if_null(text, host);
	text = (GtkEntry *) glade_xml_get_widget(gxml,"entryConnectionPort");
	gtk_entry_set_text_if_null(text, port_text);
	
	return obj;
}

gint ovc_ui_connection_show(OvcUiConnection *self)
{

	gint res = gtk_dialog_run (GTK_DIALOG(self->priv->widget));
	gtk_widget_destroy(GTK_WIDGET(self->priv->widget));
	return res;
}
