/***************************************************************************
 *            ovc-ui-connections.c
 *
 *  Fri Jun 23 20:22:27 2006
 *  Copyright  2006  Andrea Del Signore
 *  sejerpz@tin.it
 ***************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "ovc-ui-connections.h"
#include "ovc-ui-connection.h"

#include <libgnome/libgnome.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

static void ovc_ui_connections_class_init(OvcUiConnectionsClass *klass);
static void ovc_ui_connections_init(OvcUiConnections *sp);
static void ovc_ui_connections_finalize(GObject *object);

enum
{
	OVC_LIST_STORE_COLUMN_NAME,
	OVC_LIST_STORE_COLUMN_HOST,
	OVC_LIST_STORE_COLUMN_PORT,
	OVC_LIST_STORE_COLUMN_CONNECTION_OBJECT,
	OVC_LIST_STORE_COLUMNS,
};

struct _OvcUiConnectionsPrivate {
	/* Place Private Members Here */
	gchar *ui;
	OvcConnections *connections;
	GtkListStore *store;
	GtkWidget *dialog;
	GladeXML *gxml;
	gboolean disposed;
};

typedef struct _OvcUiConnectionsSignal OvcUiConnectionsSignal;
typedef enum _OvcUiConnectionsSignalType OvcUiConnectionsSignalType;

enum _OvcUiConnectionsSignalType {
	/* Place Signal Types Here */
	SIGNAL_TYPE_EXAMPLE,
	LAST_SIGNAL
};

struct _OvcUiConnectionsSignal {
	OvcUiConnections *object;
};

//static guint ovc_ui_connections_signals[LAST_SIGNAL] = { 0 };
static GObjectClass *parent_class = NULL;

GType
ovc_ui_connections_get_type()
{
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo our_info = {
			sizeof (OvcUiConnectionsClass),
			NULL,
			NULL,
			(GClassInitFunc)ovc_ui_connections_class_init,
			NULL,
			NULL,
			sizeof (OvcUiConnections),
			0,
			(GInstanceInitFunc)ovc_ui_connections_init,
		};

		type = g_type_register_static(G_TYPE_OBJECT, 
			"OvcUiConnections", &our_info, 0);
	}

	return type;
}

static void
ovc_connection_set_property (GObject * object,
							 guint property_id,
							 const GValue * value, GParamSpec * pspec)
{
	OvcUiConnections *self = (OvcUiConnections *) object;

	switch (property_id)
	{
	case OVC_UICONNECTIONS_CONSTRUCT_CONNECTIONS:
		if (self->priv->connections != NULL)
			g_object_unref (OVC_CONNECTIONS(self->priv->connections));
		self->priv->connections = OVC_CONNECTIONS(g_value_dup_object(value));
	break;
	case OVC_UICONNECTIONS_CONSTRUCT_GLADE_UI:
		g_free(self->priv->ui);		
		self->priv->ui = g_value_dup_string(value);
	break;
	default:
		/* We don't have any other property... */
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}

static void
ovc_connection_get_property (GObject * object,
							 guint property_id,
							 GValue * value, GParamSpec * pspec)
{
	OvcUiConnections *self = (OvcUiConnections *) object;

	switch (property_id)
	{
	case OVC_UICONNECTIONS_CONSTRUCT_CONNECTIONS:
		g_value_set_object (value, self->priv->connections);
	break;
	case OVC_UICONNECTIONS_CONSTRUCT_GLADE_UI:
		g_value_set_string(value, self->priv->ui);
	break;
	default:
		/* We don't have any other property... */
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
	break;
	}
}

static void
ovc_ui_connections_dispose(GObject *object)
{
	OvcUiConnections *cobj;
	cobj = OVC_UICONNECTIONS(object);
	
	if (cobj->priv->disposed == TRUE)
		return;
	
	cobj->priv->disposed = TRUE;
	
	/* Free private members, etc. */
	
	g_free(cobj->priv->ui);
	g_object_unref(cobj->priv->store);
	g_object_unref(cobj->priv->connections);
	G_OBJECT_CLASS(parent_class)->dispose(object);
}

static void
ovc_ui_connections_class_init(OvcUiConnectionsClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS(klass);
	GParamSpec *connection_param_spec;
	
	parent_class = g_type_class_peek_parent(klass);
	object_class->finalize = ovc_ui_connections_finalize;
	object_class->dispose = ovc_ui_connections_dispose;
	
	/* Create signals here:
	   ovc_ui_connections_signals[SIGNAL_TYPE_EXAMPLE] = g_signal_new(...)
 	*/

	object_class->set_property = ovc_connection_set_property;
	object_class->get_property = ovc_connection_get_property;

	connection_param_spec = g_param_spec_object("connections",
											 "Connections construct prop",
											 "Set Ovc connections object",
											 OVC_TYPE_CONNECTIONS ,
											 G_PARAM_CONSTRUCT_ONLY |
											 G_PARAM_READWRITE);

	g_object_class_install_property (object_class,
								 OVC_UICONNECTIONS_CONSTRUCT_CONNECTIONS,
								 connection_param_spec);
									 
	connection_param_spec = g_param_spec_string("glade-ui",
											 "Dialog widget construct prop",
											 "Set Ovc dialog widget object",
											"glade-ui-not-set",
											 G_PARAM_CONSTRUCT_ONLY |
											 G_PARAM_READWRITE);
											 
	g_object_class_install_property (object_class,
									 OVC_UICONNECTIONS_CONSTRUCT_GLADE_UI,
									 connection_param_spec);
}

static void
ovc_ui_connections_init(OvcUiConnections *obj)
{
	obj->priv = g_new0(OvcUiConnectionsPrivate, 1);
	/* Initialize private members, etc. */
	obj->priv->store = gtk_list_store_new (OVC_LIST_STORE_COLUMNS, 
										   G_TYPE_STRING, 
										   G_TYPE_STRING, 
										   G_TYPE_INT, 
										   OVC_TYPE_CONNECTION);
}

static void
ovc_ui_connections_finalize(GObject *object)
{
	OvcUiConnections *cobj;
	cobj = OVC_UICONNECTIONS(object);
	
	/* Free private members, etc. */
	
	g_free(cobj->priv);
	G_OBJECT_CLASS(parent_class)->finalize(object);
}

static void
store_modify_item(GtkListStore *store, GtkTreeIter *iter, OvcConnection *connection)
{
	gchar *name, *host;
	gint port=0;
	g_object_get(OVC_CONNECTION(connection), "name", &name,"host", &host, "port", &port, NULL);
	gtk_list_store_set (store, iter,
					OVC_LIST_STORE_COLUMN_NAME, name,
					OVC_LIST_STORE_COLUMN_HOST, host,
					OVC_LIST_STORE_COLUMN_PORT, port,
					OVC_LIST_STORE_COLUMN_CONNECTION_OBJECT, connection,
					-1);
	g_free(name);
	g_free(host);
}

static void
store_add_item(GtkListStore *store, OvcConnection *connection)
{
	GtkTreeIter iter;
	gtk_list_store_append (store, &iter);  /* Acquire an iterator */
	store_modify_item(store, &iter, connection);
}

static void
store_remove_item(GtkListStore *store, OvcConnection *connection)
{
	GtkTreeIter iter;
	gboolean valid;

	/* Get the first iter in the list */
	valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL(store), &iter);

	while (valid)
    {
		OvcConnection *conn;
		
		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, OVC_LIST_STORE_COLUMN_CONNECTION_OBJECT, &conn,-1);
		if (conn == connection)
		{
			gtk_list_store_remove(store, &iter);
			break;
		}
		valid = gtk_tree_model_iter_next (GTK_TREE_MODEL(store), &iter);
    }	
}

static void
connection_add(OvcUiConnections *self, OvcConnection *connection)
{
	OvcUiConnection *ui_connection = ovc_ui_connection_new(self->priv->ui, connection);
	if (ovc_ui_connection_show(ui_connection) == GTK_RESPONSE_OK)
		ovc_connections_add(self->priv->connections, connection);
}

static void
connection_modify(OvcUiConnections *self, OvcConnection *connection)
{
	OvcUiConnection *ui_connection = ovc_ui_connection_new(self->priv->ui, connection);
	ovc_ui_connection_show(ui_connection);
}

static void 
on_buttonConnectionsAdd_clicked(GtkWidget *widget, gpointer data)
{
	OvcUiConnections *connections = OVC_UICONNECTIONS(data);
	OvcConnection *connection = ovc_connection_new("");
	connection_add(connections, connection);
}

static 
OvcConnection *get_selected_connection(GtkTreeView *tree, GtkTreeIter *iter)
{
	GtkTreeSelection *select = gtk_tree_view_get_selection(tree);
	GtkTreeModel *store;
	if (gtk_tree_selection_get_selected(select, &store, iter))
	{
		OvcConnection *conn;
		gtk_tree_model_get(store, iter, OVC_LIST_STORE_COLUMN_CONNECTION_OBJECT, &conn,-1);
		return conn;
	}
	else
		return NULL;
}

static void 
on_buttonConnectionsRemove_clicked(GtkWidget *widget, gpointer data)
{
	OvcUiConnections *self = (OvcUiConnections *) data;
	OvcConnection *connection;
	GtkTreeView *tree = (GtkTreeView *) glade_xml_get_widget(self->priv->gxml, "treeviewConnections");
	GtkTreeIter iter;
	
	if ( (connection = get_selected_connection(tree, &iter)) != NULL)
	{
		GladeXML *gxml = glade_xml_new (self->priv->ui, "dialogConnectionRemove", NULL);
		GtkDialog *confirm = (GtkDialog *) glade_xml_get_widget(gxml, "dialogConnectionRemove");
		
		if (gtk_dialog_run(confirm) == GTK_RESPONSE_OK)	
		{
			ovc_connections_remove(self->priv->connections, connection);
		}
		gtk_widget_destroy(GTK_WIDGET(confirm));
		g_object_unref(gxml);
	}	
}


static
void on_buttonConnectionsModify_clicked(GtkWidget *widget, gpointer data)
{
	OvcUiConnections *connections = OVC_UICONNECTIONS(data);
	OvcConnection *connection;
	GtkTreeView *tree = (GtkTreeView *) glade_xml_get_widget(connections->priv->gxml, "treeviewConnections");
	GtkTreeIter iter;
	
	if ( (connection = get_selected_connection(tree, &iter)) != NULL)
	{
		connection_modify(connections, connection);
		/* update ui */
		store_modify_item(connections->priv->store, &iter, connection);
	}
}


static
void on_buttonConnectionsClose_clicked(GtkWidget *widget, gpointer data)
{
	OvcUiConnections *obj = OVC_UICONNECTIONS(data);
	
	//gtk_dialog_response(obj->priv->dialog, GTK_RESPONSE_CLOSE);
	gtk_widget_destroy(obj->priv->dialog);
	
	/* CONNECTIONS */
	gchar *configfile = gnome_util_home_file("ovpnclient.xml");
	ovc_connections_save(obj->priv->connections, configfile);
}




static void
on_list_clear(OvcConnections *sender, gpointer data)
{
	OvcUiConnections *self = (OvcUiConnections *) data;
	
	gtk_list_store_clear(self->priv->store);
}

static void
on_item_added(OvcConnections *sender, OvcConnection *connection, gpointer data)
{
	OvcUiConnections *self = (OvcUiConnections *) data;
	store_add_item(self->priv->store,connection);
}

static void
on_item_removed(OvcConnections *sender, OvcConnection *connection, gpointer data)
{
	OvcUiConnections *self = (OvcUiConnections *) data;
	store_remove_item(self->priv->store, connection);
}



OvcUiConnections *
ovc_ui_connections_new(const gchar *glade_ui, OvcConnections *connections)
{
	OvcUiConnections *obj;
	
	obj = OVC_UICONNECTIONS(g_object_new(OVC_TYPE_UICONNECTIONS, "glade-ui", glade_ui, "connections", connections, NULL));

	g_signal_connect (G_OBJECT (connections), "list-clear", G_CALLBACK (on_list_clear), (gpointer) obj);
	g_signal_connect (G_OBJECT (connections), "item-added", G_CALLBACK (on_item_added), (gpointer) obj);
	g_signal_connect (G_OBJECT (connections), "item-removed", G_CALLBACK (on_item_removed), (gpointer) obj);

	GladeXML *gxml = glade_xml_new (glade_ui, "windowConnections", NULL);
	obj->priv->gxml = gxml;
	GList *item;
	
	
	for(g_object_get(OVC_CONNECTIONS(connections), "items", &item, NULL); item != NULL; item = item->next)
	{
		OvcConnection *connection = OVC_CONNECTION(item->data);	
		store_add_item(obj->priv->store, connection);
	}
	
	GtkTreeView *tree = GTK_TREE_VIEW(glade_xml_get_widget(gxml, "treeviewConnections"));
	gtk_tree_view_set_model(tree, GTK_TREE_MODEL(obj->priv->store));
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	
	/* Setup the selection handler */
	GtkTreeSelection *select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
	
	
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Name",
                                                   renderer,
                                                   "text", 0,
                                                   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Host",
                                                   renderer,
                                                   "text", 1,
                                                   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Port",
                                                   renderer,
                                                   "text", 2,
                                                   NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);

	/* Dialog event setup */
	GtkWidget *dialog = glade_xml_get_widget(gxml, "windowConnections");
	obj->priv->dialog = dialog;
	
	GtkButton *button;
	
	button= GTK_BUTTON(glade_xml_get_widget(gxml, "buttonConnectionsAdd"));
	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_buttonConnectionsAdd_clicked), (gpointer) obj);
	
	button = GTK_BUTTON(glade_xml_get_widget(gxml, "buttonConnectionsRemove"));
	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_buttonConnectionsRemove_clicked), (gpointer) obj);
	
	button = GTK_BUTTON(glade_xml_get_widget(gxml, "buttonConnectionsModify"));
	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_buttonConnectionsModify_clicked), (gpointer) obj);
	
	button = GTK_BUTTON(glade_xml_get_widget(gxml, "buttonConnectionsClose"));
	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_buttonConnectionsClose_clicked), (gpointer) obj);
	
	return obj;
}

void ovc_ui_connections_show(OvcUiConnections *self)
{
	gtk_widget_show(self->priv->dialog);
}
